#! /usr/bin/env python3.4

import tkinter as tk
from tkinter import filedialog
from PIL import ImageTk, Image

COLORS = {1: 'red', 2: 'green', 3: 'blue', 4: 'white', 5: 'yellow', 6: 'violet'}
gems_map = []
marked = None
coords = [0, 0]

def get_sleds_score(gems, pos_i, pos_j):
    score = 0
    n = 1
    for i in range(pos_i+1, 8):
        if gems[i][pos_j] == gems[pos_i][pos_j]:
            n += 1
        else:
            break
    for i in range(pos_i-1, -1, -1):
        if gems[i][pos_j] == gems[pos_i][pos_j]:
            n += 1
        else:
            break
    if n > 2:
        score = (n - 2) * 3
    n = 1
    for j in range(pos_j+1, 8):
        if gems[pos_i][j] == gems[pos_i][pos_j]:
            n += 1
        else:
            break
    for j in range(pos_j-1, -1, -1):
        if gems[pos_i][j] == gems[pos_i][pos_j]:
            n += 1
        else:
            break
    if n > 2:
        return score + ((n - 2) * 3)
    else:
        return score

def render_gems(lines):
    i = 0
    gems = []
    for line in lines[:8]:
        gems.append([])
        j = 0
        for char in line:
            gems[int(i/50)].append(int(char))
            oval = canvas.create_oval(j+10, i+10, j+40, i+40, fill=COLORS[int(char)], width=2, tags = ("r"+str(i/50),"s"+str(j/50)))
            canvas.tag_bind(oval, '<ButtonPress-1>', gem_click)
            j += 50
        i += 50
    return gems

def get_switchable_gems(gems):
    switchable = []
    for i in range(8):
        for j in range(8):
            if i+1 < 8:
                gems[i][j], gems[i+1][j] = gems[i+1][j], gems[i][j]
                score = get_sleds_score(gems, i, j) + get_sleds_score(gems, i+1, j)
                gems[i][j], gems[i+1][j] = gems[i+1][j], gems[i][j]
                if score >= 3:
                    switchable.append((i, j, i+1, j))
            if j+1 < 8:
                gems[i][j], gems[i][j+1] = gems[i][j+1], gems[i][j]
                score = get_sleds_score(gems, i, j) + get_sleds_score(gems, i, j+1)
                gems[i][j], gems[i][j+1] = gems[i][j+1], gems[i][j]
                if score >= 3:
                    switchable.append((i, j, i, j+1))
    return switchable

def gem_click(event):
    item = event.widget.find_closest(event.x, event.y)[0]
    canvas.itemconfig(item, outline='orange')
    if marked is not None:
        coords2 = [0, 0]
        for tag in canvas.gettags(item):
            if tag[0] == "r":
                coords2[0] = int(tag[1])
            if tag[0] == "s":
                coords2[1] = int(tag[1])
        switchable = get_switchable_gems(gems_map)
        if (coords[0], coords[1], coords2[0], coords2[1]) in switchable or \
           (coords2[0], coords2[1], coords[0], coords[1]) in switchable:
            pass
    else:
        global marked
        marked = item
        global coords
        coords = [0, 0]
        for tag in canvas.gettags(item):
            if tag[0] == "r":
                coords[0] = int(tag[1])
            if tag[0] == "s":
                coords[1] = int(tag[1])

def button_click():
    file_path_string = tk.filedialog.askopenfilename()
    with open(file_path_string, 'r') as f:
        lines = f.read().splitlines()

    global gems_map
    gems_map = render_gems(lines)
    incoming_gems = lines[8:]


master = tk.Tk()

frame = tk.Frame(master, bg='grey', width=400, height=100)
frame.pack(fill='x')
startButton = tk.Button(frame, command=button_click, text="Start")
startButton.pack(side='left', padx=10)

canvas = tk.Canvas(master, width=400, height=400)
canvas.pack()

for i in range(0, 400, 50):
    for j in range(0, 400, 50):
        canvas.create_oval(i+10, j+10, i+40, j+40, width=2)

tk.mainloop()
